﻿using System;
using System.IO;

namespace NetworkChallengedStreams
{
    class LoggingStream : Stream
    {
        private readonly Stream _source;

        public LoggingStream(Stream source)
        {
            _source = source;
        }

        public override bool CanRead
        {
            get { return _source.CanRead; }
        }

        public override bool CanSeek
        {
            get { return _source.CanSeek; }
        }

        public override bool CanWrite
        {
            get { return _source.CanWrite; }
        }

        public override void Flush()
        {
            Console.WriteLine();
            Console.WriteLine("[Flush]");
            _source.Flush();
        }

        public override long Length
        {
            get { return _source.Length; }
        }

        public override long Position
        {
            get
            {
                return _source.Position;
            }
            set
            {
                Console.WriteLine();
                Console.WriteLine("[Position={0}]", value);
                _source.Position = value;
            }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            Console.WriteLine();
            Console.WriteLine("[BinRead requested {0} bytes]", count);
            int result = _source.Read(buffer, offset, count);
            Console.WriteLine("[BinRead returns {0} bytes]", result);
            return result;
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            return _source.Seek(offset, origin);
        }

        public override void SetLength(long value)
        {
            _source.SetLength(value);
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            _source.Write(buffer, offset, count);
        }
    }
}
