﻿using System;
using System.IO;

namespace NetworkChallengedStreams
{
    class LoggingTextReader : TextReader
    {
        private readonly TextReader _source;

        public LoggingTextReader(TextReader source)
        {
            _source = source;
        }

        public override void Close()
        {
            _source.Close();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                _source.Dispose();
            }
        }

        public override int Peek()
        {
            return _source.Peek();
        }

        public override int Read()
        {
            Console.WriteLine();
            Console.WriteLine("[TextRead]");

            var result = _source.Read();
            if (result == -1)
            {
                Console.WriteLine();
                Console.WriteLine("[TextRead returns EOF]");
            }
            else
            {
                Console.WriteLine();
                Console.WriteLine("[TextRead returns " + (char)result + "]");
            }

            return result;
        }

        public override int Read(char[] buffer, int index, int count)
        {
            Console.WriteLine();
            Console.WriteLine("[TextRead requested " + count + " chars]");
            int result = _source.Read(buffer, index, count);
            if (result == 0)
            {
                Console.WriteLine();
                Console.WriteLine("[TextRead returns 0 chars]");
            }
            else
            {
                Console.WriteLine();
                Console.WriteLine("[TextRead returns " + result + " chars]");
            }

            return result;
        }
    }
}
